/*
 * Created with SharpDevelop.
 * User: Vincent
 * Date: 04/01/2005
 * Time: 18:39
 */
// one line to give the program's name and an idea of what it does.
// Copyright (C) yyyy  name of author
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Windows.Forms;

namespace Client
{
	/// <summary>
	/// Description of ClientClass.
	/// </summary>
	public class ClientClass
	{
		private Socket SocketClient;
		private bool LocalsocketClientIsShutingDown;
		private byte[] readbuf;
		private byte[] sendbuf;
		
		private ListBox listBox;
		
		public delegate void DelegateDisplay(string message);
		
		public ClientClass(System.Windows.Forms.ListBox list)
		{
			this.listBox = list;
			readbuf = new byte[100];
		}
		
		/// <summary>
		/// Fonction de connection au serveur
		/// </summary>
		/// <param name="ServerName">Le nom du serveur auquel on veux se connecter</param>
		public void ConnectToServer(string ServerName)
		{
			if (this.SocketClient == null || !this.SocketClient.Connected)
			{
				this.SocketClient = new Socket(AddressFamily.InterNetwork,SocketType.Stream,ProtocolType.Tcp);

				IPAddress[] ipadress;
				//On rcupere les informations du serveur (son nom puis son adresse IP)
				IPHostEntry he = Dns.GetHostByName(ServerName);
				ipadress = he.AddressList;
			 	
				this.SocketClient.BeginConnect(new IPEndPoint(ipadress[0],15),new AsyncCallback(ConnectCallback),this.SocketClient);
			}
			else
			{
				//on affiche un message
				string[] obj = new string[1] {"Dj connect  un serveur"};
				this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);				
			}

		}
		
		/// <summary>
		/// Fonction de rappel pour la connection
		/// </summary>
		/// <param name="asyncResult">un objet IAsyncResult</param>
		private void ConnectCallback(IAsyncResult asyncResult)
		{
			string[] obj;
			try
			{
				//on rcupere le socket sur lequel la connexion doit avoir lieu
				Socket socket = (Socket)asyncResult.AsyncState;
				//On affect SocketClient au socket de connection
				this.SocketClient = socket;
				//on met fin  la demande de connection
				socket.EndConnect(asyncResult);
				
				//on affiche un message
				obj = new string[1] {"Connect  un serveur"};
				this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);
				this.LocalsocketClientIsShutingDown = false;

				// On initie une lecture
				this.SocketClient.BeginReceive(this.readbuf,0,this.readbuf.Length,SocketFlags.None,new AsyncCallback(ReceiveCallback),this.SocketClient);
			}
			catch (SocketException ex)
			{
				obj = new string[1] {ex.Message};
				this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);
			}
		}
		
		/// <summary>
		/// Fonction qui permet l'envois de message
		/// </summary>
		/// <param name="message"></param>
		public void SendMessage(string message)
		{
			if (this.SocketClient != null && this.SocketClient.Connected)
			{
				//Transformation de la string en byte pour transfert par le rseaux
				this.sendbuf = Encoding.ASCII.GetBytes(message);
				//on envois le tout.
				this.SocketClient.BeginSend(this.sendbuf,0,this.sendbuf.Length,SocketFlags.None,new AsyncCallback(SendCallback),this.SocketClient);
			}
			else
				DisplayMessage("Non connect au serveur.");
		}
		
		/// <summary>
		/// Fonction de rappel pour la fonction d'envois de message
		/// </summary>
		/// <param name="asyncResult">Un objet IAsyncResult</param>
		private void SendCallback(IAsyncResult asyncResult)
		{
			string[] obj;

			try
			{
				//On rcupere le socket sur lequel on a envoy les donnes
				Socket socket = (Socket)asyncResult.AsyncState;
				//on met fin  l'envois de donnes
				int send = socket.EndSend(asyncResult);
				
				//ajoute le message
				obj = new string[1] {"Message envoy (" + send + " bytes envoyes )"};
				this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);
			}
			catch (SocketException ex)
			{
				obj = new string[1] {ex.Message};
				this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);
			}
		}
		
		/// <summary>
		/// Fonction qui lance l'attente de message
		/// </summary>
		public void ReceiveMessage()
		{

			if (this.SocketClient != null && this.SocketClient.Connected)
				this.SocketClient.BeginReceive(this.readbuf,0,this.readbuf.Length,SocketFlags.None,new AsyncCallback(ReceiveCallback),this.SocketClient);
			else
				DisplayMessage("Non connect au serveur.");

		}
		
		/// <summary>
		/// Fonction de rappel pour la rception d'un message
		/// </summary>
		/// <param name="asyncResult">Un objet IAsyncResult transmit par BeginReceive</param>
		private void ReceiveCallback(IAsyncResult asyncResult)
		{
			string[] obj;

			try
			{
				//On rcupere le socket de la connection
				Socket socket = (Socket)asyncResult.AsyncState;
				
				//on lis les donnes sur le socket
				int read = socket.EndReceive(asyncResult);
							
				//si on lis plus de 0 octet alors c'est que tout le message a t transmis
				if( read > 0 )
				{
					//ajoute le message
					obj = new string[] {"Serveur dit :" + Encoding.ASCII.GetString(this.readbuf)};
					this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);
					// reset buffer
					Buffer.SetByte(this.readbuf, 0, 0);
					// On initie une lecture
					this.SocketClient.BeginReceive(this.readbuf,0,this.readbuf.Length,SocketFlags.None,new AsyncCallback(ReceiveCallback),this.SocketClient);
				}
				//Si 0 octets est transmis alors on doit fermer la connection
				if( read == 0 && !this.LocalsocketClientIsShutingDown)
				{
					this.SocketClient.Close();
					obj = new string[] {"Fermeture socket distante"};
					this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);
				}
			}
			catch (SocketException ex)
			{
				obj = new string[1] {ex.Message};
				this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);
			}
		}
		
		/// <summary>
		/// Fonction qui ferme correctement la communication
		/// </summary>
		public void Close()
		{
			if (this.SocketClient !=null && this.SocketClient.Connected)
			{
				this.LocalsocketClientIsShutingDown = true;
				//On ferme le socket
				this.SocketClient.Shutdown(SocketShutdown.Both);
				//System.Threading.Thread.Sleep(1000);
				
				//On dtruit le socket
				this.SocketClient.Close();
				
				//On ajoute le message  la listbox .
				string[] obj = new string[1] {"Connexion ferme"};
				this.listBox.Invoke(new DelegateDisplay(this.DisplayMessage),obj);
			}
		}
		
		/// <summary>
		/// Fonction qui permet l'ajout d'un message dans la listbox dans le bon thread
		/// </summary>
		/// <param name="message">Le message  afficher</param>
		private void DisplayMessage(string message)
		{
			this.listBox.Items.Add(message);
			this.listBox.SelectedIndex = this.listBox.Items.Count - 1;
		}
		
	}
}
